# Copyright (c) 2008 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from AccessControl.requestmethod import postonly
from Globals import InitializeClass


from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.Silva.helpers import add_and_edit

SERVICE_ID = 'service_addablespermissions'

class AddablesPermissionsService(SimpleItem):
    """Service to configure which content type is addable.
    """

    meta_type = 'Silva Addables Permissions Service'
    title = 'Silva Addables Permissions Service'

    security = ClassSecurityInfo()

    manage_options = (
        {'label':'Edit', 'action':'manage_editForm'},
        ) + SimpleItem.manage_options
    
    security.declareProtected('View management screens', 
                              'manage_editForm')
    manage_editForm = PageTemplateFile(
        'www/addablesPermissionsServiceEdit', globals(),  
        __name__='manage_editForm')


    security.declareProtected('View management screens', 
                              'manage_editAddablesPermissions')
    @postonly
    def manage_editAddablesPermissions(self, REQUEST):
        """Edit permissions.
        """
        root = self.get_root()
        for type in root.get_silva_addables_all():
            permission = 'Add ' + type + 's'
            current = root.rolesOfPermission(permission)
            current = [r['name'] for r in current if r['selected']]
            wanted = REQUEST.form[type]
            authorized = False
            for role in self.manageableRoles():
                if role == wanted:
                    authorized = True
                if (not authorized) and role in current:
                    current.remove(role)
                if authorized and not (role in current):
                    current.append(role)
            root.manage_permission(permission, current, 0)


    security.declareProtected('View management screens', 
                              'manageableRoles')
    def manageableRoles(self):
        """Return roles for which we manage permissions.
        """
        root = self.get_root()
        return root.sec_get_roles()[4:]

    security.declareProtected('View management screens', 
                              'currentAddablesPermissions')
    def currentAddablesPermissions(self):
        """Return current permission settings.
        """
        root = self.get_root()
        settings = {}
        for type in root.get_silva_addables_all():

            permissions = root.rolesOfPermission('Add ' + type + 's')
            permissions = dict([(r['name'], '' != r['selected']) for r in permissions])
            winner = None
            for role in self.manageableRoles():
                if permissions[role]:
                    if winner is None:
                        winner = role
                elif not (winner is None): # Sanaty check
                        msg = 'Invalid permission settings. %s can add %s but %s cannot.'
                        raise ValueError, msg % (winner, type, role)

            settings[type] = winner
        return settings

    def __init__(self, id):
        self.id = id


InitializeClass(AddablesPermissionsService)

manage_addAddablesPermissionsServiceForm = PageTemplateFile(
       "www/addablesPermissionsServiceAdd",  globals(),
       __name__ = 'manage_addAddablesPermissionsServiceForm')

def manage_addAddablesPermissionsService(self, id, REQUEST=None):
    """Add the security addable service.
    """
    id = self._setObject(id, AddablesPermissionsService(id))
    add_and_edit(self, id, REQUEST)
    return ''



# Copyright (c) 2008 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from AddablesPermissions import SERVICE_ID

def install(root):
    """Install method for the service.
    """
    product = root.manage_addProduct['silva.security.addables']
    product.manage_addAddablesPermissionsService(SERVICE_ID)

def uninstall(root):
    """Uninstall method for the service.
    """
    root.manage_delObjects([SERVICE_ID])

def is_installed(root):
    """Return true is the service is installed.
    """
    return hasattr(root.aq_base, SERVICE_ID)


if __name__ == '__main__':
    print """This module is not an installer. You don't have to run it."""



# Copyright (c) 2008 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import install
import AddablesPermissions

from Products.Silva.ExtensionRegistry import extensionRegistry

def initialize(context):
    extensionRegistry.register(
        'silva.security.addables', 
        'Silva configuration utility for addables permissions', 
        context, [], install, depends_on='Silva')

    context.registerClass(
        AddablesPermissions.AddablesPermissionsService,
        constructors = (AddablesPermissions.manage_addAddablesPermissionsServiceForm,
                        AddablesPermissions.manage_addAddablesPermissionsService),
        icon = "www/addablespermissions_service.gif"
        )




# Copyright (c) 2008 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from Products.Silva.tests import SilvaTestCase
from Products.Five import zcml

from Testing import ZopeTestCase as ztc
from Testing.ZopeTestCase.layer import onsetup as ZopeLiteLayerSetup

@ZopeLiteLayerSetup
def installPackage(name):
    """This used to be executed at start time, but not anymore ...
    (see Silva 2.2 test setup).

    This is required for Zope 2.11.
    """
    ztc.installPackage(name)

def haveSilvaFind(root):
    """Test if SilvaFind is available.
    """
    service  = root.service_extensions
    return service.is_installed('SilvaFind')


class FakeRequest(dict):
    """Dictionarish like fake request with an attribute form.
    """

    @property
    def form(self):
        return self['form']

class AddablesPermissionsTestCase(SilvaTestCase.SilvaTestCase):
    """Tests for silva.security.addables.
    """

    def afterSetUp(self):
        """After set up, install the extension.
        """
        root = self.getRoot()
        root.service_extensions.install('silva.security.addables')

    def test_00install(self):
        """Install should provide a new service.
        """
        root = self.getRoot()
        service_ext = root.service_extensions
        self.failUnless(service_ext.is_installed('silva.security.addables'))
        self.failUnless(hasattr(root.aq_base, 'service_addablespermissions'))

    def test_10retrieve(self):
        """There is utilities which retrieve current settings.
        """
        root = self.getRoot()
        service = root.service_addablespermissions

        expected_roles = ('Author', 'Editor', 'ChiefEditor', 'Manager')
        self.assertEqual(service.manageableRoles(), expected_roles)

        expected_perms = {'Silva AutoTOC': 'Author',
                          'Silva Link': 'Author',
                          'Silva Folder': 'Author',
                          'Silva Image': 'Author',
                          'Silva Publication': 'Editor',
                          'Silva Document': 'Author',
                          'Silva File': 'Author',
                          'Silva Ghost Folder': 'Editor',
                          'Silva Indexer': 'Editor',
                          'Silva Ghost': 'Author'}
        if haveSilvaFind(root):
            expected_perms['Silva Find'] = 'Editor'

        self.assertEqual(service.currentAddablesPermissions(), expected_perms)


    def test_20modification(self):
        """Modification should work.
        """
        root = self.getRoot()
        service = root.service_addablespermissions

        new_perms = {'Silva AutoTOC': 'Manager',
                     'Silva Link': 'Author',
                     'Silva Folder': 'Author',
                     'Silva Image': 'Author',
                     'Silva Publication': 'Editor',
                     'Silva Document': 'Author',
                     'Silva File': 'Author',
                     'Silva Ghost Folder': 'Editor',
                     'Silva Indexer': 'ChiefEditor',
                     'Silva Ghost': 'Author'}
        if haveSilvaFind(root):
            new_perms['Silva Find'] = 'ChiefEditor'

        REQUEST = FakeRequest({'form': new_perms})
        service.manage_editAddablesPermissions(REQUEST)

        self.assertEqual(service.currentAddablesPermissions(), new_perms)

    def test_60corruptpermissions(self):
        """When permission settings doesn't match Silva logic, we
        should get an value error.
        """
        root = self.getRoot()
        service = root.service_addablespermissions

        # Set bad permissions
        root.manage_permission('Add Silva Documents',
                               ('Author', 'Manager'),
                               0)

        self.assertRaises(ValueError, service.currentAddablesPermissions)


    def test_80uninstall(self):
        """The uninstall method should remove the service.
        """
        root = self.getRoot()
        service_ext = root.service_extensions
        service_ext.uninstall('silva.security.addables')
        self.failIf(service_ext.is_installed('silva.security.addables'))
        self.failIf(hasattr(root.aq_base, 'service_addablespermissions'))



import unittest
def test_suite():

    # Load Five ZCML
    from Products import Five
    zcml.load_config('meta.zcml', Five)
    zcml.load_config('configure.zcml', Five)

    # Load our ZCML, which add the extension as a Product
    from silva.security import addables
    zcml.load_config('configure.zcml', addables)

    # Load the Zope Product
    installPackage('silva.security.addables')

    # Run tests
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(AddablesPermissionsTestCase))
    return suite
